#ifndef __NDS_GAP_HANDLER_HH__
#define __NDS_GAP_HANDLER_HH__

#include <stdint.h>

#include "nds_buffer.hh"

namespace NDS
{
  class delayed_gap_handler
  {
  public:
    delayed_gap_handler() {};
    virtual ~delayed_gap_handler() {};

    virtual void operator()() = 0;
  };

  class gap_handler
  {
  public:
  	gap_handler() {};
  	virtual ~gap_handler() {};

  	virtual delayed_gap_handler* fill_gap(buffer &cur_buffer, buffer::size_type start_sample_offset, buffer::size_type end_sample_offset) const = 0;
  };

  class fixed_point_gap_handler : public gap_handler
  {
  public:
    struct static_val {
      ::int16_t int16val;
      ::int32_t int32val;
      ::int64_t int64val;
      float float32val;
      double float64val;
      float complexrval;
      float complexival;
      ::uint32_t uint32val;

      typedef enum 
      {
        ZERO_VAL = 0,
        ONE_VAL,
        NAN_VAL,
        POS_INF_VAL,
        NEG_INF_VAL,
      } fixed_values;
      static_val(::int16_t int16val, ::int32_t int32val, ::int64_t int64val, \
        float float32val, double float64val, \
        float complexrval, float complexival, ::uint32_t uint32val);
      explicit static_val(fixed_values spec);
      explicit static_val(double value = 0.0);
    };

  public:
    fixed_point_gap_handler(static_val::fixed_values spec):val(spec) {};
  	fixed_point_gap_handler(const static_val &val):val(val) {};
  	virtual ~fixed_point_gap_handler() {};
  	virtual delayed_gap_handler* fill_gap(buffer &cur_buffer, buffer::size_type start_sample_offset, buffer::size_type end_sample_offset ) const;

  protected:
    static_val val;
  };

  class continuation_gap_handler: public fixed_point_gap_handler
  {
  public:
    continuation_gap_handler(const static_val default_val): fixed_point_gap_handler(default_val) {};
    virtual ~continuation_gap_handler() {};

    virtual delayed_gap_handler* fill_gap(buffer &cur_buffer, buffer::size_type start_sample_offset, buffer::size_type end_sample_offset ) const;
  };

  class abort_gap_handler : public gap_handler
  {
  public:
  	abort_gap_handler() {};
  	virtual ~abort_gap_handler() {};
  	virtual delayed_gap_handler* fill_gap(buffer &cur_buffer, buffer::size_type start_sample_offset, buffer::size_type end_sample_offset ) const;
  };
}

#endif // __NDS_GAP_HANDLER_HH__