#include "daqc.h"
#include "nds_helper.hh"

//#include "debug_stream.hh"

#include <algorithm>

namespace NDS
{
  channel
  create_channel( const chan_req_t& Source )
  {
    channel	retval( Source.name,
			channel::convert_daq_chantype( Source.type ),
			channel::convert_daq_datatype( Source.data_type ),
			Source.rate,
			Source.s.signal_gain,
			Source.s.signal_slope,
			Source.s.signal_offset,
			Source.s.signal_units );

    return retval;
  }

  bool availability_comp( const segment& a, const segment& b )
  {
  	if (a.gps_start < b.gps_start) return true;
  	if (a.gps_start == b.gps_start)
  	{
  		return (a.gps_stop > b.gps_stop);
  	}
  	return false;
  }

  simple_segment_list_type simplify_availability(const availability& avail)
  {
      availability sortlist(avail);
      std::sort(sortlist.data.begin(), sortlist.data.end(), availability_comp);
      simple_segment_list_type retval;

      buffer::gps_second_type gps_cur = 0;
      for (segment_list_type::iterator cur = sortlist.data.begin(); cur != sortlist.data.end(); ++cur)
      {
          buffer::gps_second_type start = cur->gps_start, stop = cur->gps_stop;
          //NDS::dout << "ex: cur = " << gps_cur << " " << cur->frame_type << " " << start << " " << stop << std::endl;
          if (gps_cur >= stop) { continue; }

          if (gps_cur > start) { start = gps_cur; }

          if (gps_cur == start && retval.size() > 0) {
              retval.back().gps_stop = stop;
          } else {
              retval.push_back(simple_segment( start, stop ));
          }
          gps_cur = stop;
      }
      return retval;
  }

  simple_availability_list_type simplify_availability_list(const availability_list_type &avails)
  {
      simple_availability_list_type results;
      results.resize(avails.size());
      std::transform(avails.begin(), avails.end(), results.begin(), simplify_availability);
      return results;
  }
}
